@@ -1,8 +1,6 @@
 from abc import abstractmethod
 from typing import List
 
-from django.views import View
-
 from drf_spectacular.plumbing import OpenApiGeneratorExtension
 
 
@@ -11,12 +9,12 @@ class OpenApiAuthenticationExtension(OpenApiGeneratorExtension['OpenApiAuthentic
 
     name: str
 
-    def get_security_requirement(self, view: View):
+    def get_security_requirement(self, auto_schema):
         assert self.name, 'name must be specified'
         return {self.name: []}
 
     @abstractmethod
-    def get_security_definition(self, view: View):
+    def get_security_definition(self, auto_schema):
         pass
 
 
@@ -24,7 +22,7 @@ class SessionScheme(OpenApiAuthenticationExtension):
     target_class = 'rest_framework.authentication.SessionAuthentication'
     name = 'cookieAuth'
 
-    def get_security_definition(self, view: View):
+    def get_security_definition(self, auto_schema):
         return {
             'type': 'apiKey',
             'in': 'cookie',
@@ -36,7 +34,7 @@ class BasicScheme(OpenApiAuthenticationExtension):
     target_class = 'rest_framework.authentication.BasicAuthentication'
     name = 'basicAuth'
 
-    def get_security_definition(self, view: View):
+    def get_security_definition(self, auto_schema):
         return {
             'type': 'http',
             'scheme': 'basic',
@@ -48,7 +46,7 @@ class TokenScheme(OpenApiAuthenticationExtension):
     name = 'tokenAuth'
     matches_subclass = True
 
-    def get_security_definition(self, view: View):
+    def get_security_definition(self, auto_schema):
         return {
             'type': 'http',
             'scheme': 'bearer',
