@@ -1,3 +1,4 @@
+from drf_spectacular.plumbing import warn
 from drf_spectacular.serializers import OpenApiSerializerExtension
 
 
@@ -5,15 +6,22 @@ class PolymorphicSerializerExtension(OpenApiSerializerExtension):
     target_class = 'rest_polymorphic.serializers.PolymorphicSerializer'
     match_subclasses = True
 
-    def map_serializer(self, auto_schema, method: str):
+    def map_serializer(self, auto_schema, direction):
         sub_components = []
         serializer = self.target
 
-        for sub_model, sub_serializer in serializer.model_serializer_mapping.items():
+        for sub_model in serializer.model_serializer_mapping:
+            sub_serializer = serializer._get_serializer_from_model_or_instance(sub_model)
             resource_type = serializer.to_resource_type(sub_model)
-            ref = auto_schema.resolve_serializer(method, sub_serializer).ref
+            ref = auto_schema.resolve_serializer(sub_serializer, direction).ref
             sub_components.append((resource_type, ref))
 
+            if not resource_type:
+                warn(
+                    f'discriminator mapping key is empty for {sub_serializer.__class__}. '
+                    f'this might lead to code generation issues.'
+                )
+
         return {
             'oneOf': [ref for _, ref in sub_components],
             'discriminator': {
