@@ -4,27 +4,27 @@ import typing
 from collections import OrderedDict
 from decimal import Decimal
 from operator import attrgetter
-from urllib.parse import urljoin
 
 import uritemplate
 from django.core import validators, exceptions as django_exceptions
 from django.db import models
 from django.utils.encoding import force_str
-from rest_framework import permissions, renderers, serializers, views, viewsets
+from rest_framework import permissions, renderers, serializers
 from rest_framework.fields import _UnvalidatedField, empty
 from rest_framework.generics import GenericAPIView
-from rest_framework.schemas.generators import BaseSchemaGenerator
 from rest_framework.schemas.inspectors import ViewInspector
-from rest_framework.schemas.utils import get_pk_description, is_list_view
+from rest_framework.schemas.utils import get_pk_description
+from rest_framework.settings import api_settings
+from rest_framework.views import APIView
 
+from drf_spectacular.fields import OpenApiSerializerFieldExtension
 from drf_spectacular.settings import spectacular_settings
 from drf_spectacular.contrib.authentication import *  # noqa: F403, F401
 from drf_spectacular.contrib.serializers import *  # noqa: F403, F401
 from drf_spectacular.plumbing import (
     build_basic_type, warn, anyisinstance, force_instance, is_serializer,
-    follow_field_source, is_field, is_basic_type, alpha_operation_sorter,
-    get_field_from_model, build_array_type, ComponentRegistry, ResolvedComponent,
-    build_root_object, reset_generator_stats, build_parameter_type
+    follow_field_source, is_field, is_basic_type, build_array_type,
+    ComponentRegistry, ResolvedComponent, build_parameter_type
 )
 from drf_spectacular.types import OpenApiTypes
 from drf_spectacular.utils import OpenApiParameter
@@ -32,83 +32,6 @@ from drf_spectacular.authentication import OpenApiAuthenticationExtension
 from drf_spectacular.serializers import OpenApiSerializerExtension
 
 
-class SchemaGenerator(BaseSchemaGenerator):
-    def __init__(self, *args, **kwargs):
-        self.registry = ComponentRegistry()
-        super().__init__(*args, **kwargs)
-
-    def create_view(self, callback, method, request=None):
-        """
-        customized create_view which is called when all routes are traversed. part of this
-        is instatiating views with default params. in case of custom routes (@action) the
-        custom AutoSchema is injected properly through 'initkwargs' on view. However, when
-        decorating plain views like retrieve, this initialization logic is not running.
-        Therefore forcefully set the schema if @extend_schema decorator was used.
-        """
-        view = super().create_view(callback, method, request)
-
-        if isinstance(view, viewsets.GenericViewSet) or isinstance(view, viewsets.ViewSet):
-            action = getattr(view, view.action)
-        elif isinstance(view, views.APIView):
-            action = getattr(view, method.lower())
-        else:
-            warn(
-                'Using not supported View class. Class must be derived from APIView '
-                'or any of its subclasses like GenericApiView, GenericViewSet.'
-            )
-            return view
-
-        if hasattr(action, 'kwargs') and 'schema' in action.kwargs:
-            # might already be properly set in case of @action but overwrite for all cases
-            view.schema = action.kwargs['schema']
-
-        return view
-
-    def get_endpoints(self, request):
-        """ sorted endpoints by operation """
-        self._initialise_endpoints()
-        _, endpoints = self._get_paths_and_endpoints(request)
-
-        if spectacular_settings.OPERATION_SORTER == 'alpha':
-            return sorted(endpoints, key=alpha_operation_sorter)
-        else:
-            # default to DRF method sorting
-            return endpoints
-
-    def parse(self, request=None):
-        """ Iterate endpoints generating per method path operations. """
-        result = {}
-
-        for path, method, view in self.get_endpoints(request):
-            if not self.has_view_permissions(path, method, view):
-                continue
-
-            # beware that every access to schema yields a fresh object (descriptor pattern)
-            operation = view.schema.get_operation(path, method, self.registry)
-
-            # operation was manually removed via @extend_schema
-            if not operation:
-                continue
-
-            # Normalise path for any provided mount url.
-            if path.startswith('/'):
-                path = path[1:]
-            path = urljoin(self.url or '/', path)
-
-            result.setdefault(path, {})
-            result[path][method.lower()] = operation
-
-        return result
-
-    def get_schema(self, request=None, public=False):
-        """ Generate a OpenAPI schema. """
-        reset_generator_stats()
-        return build_root_object(
-            paths=self.parse(None if public else request),
-            components=self.registry.build(spectacular_settings.APPEND_COMPONENTS),
-        )
-
-
 class AutoSchema(ViewInspector):
     method_mapping = {
         'get': 'retrieve',
@@ -120,47 +43,81 @@ class AutoSchema(ViewInspector):
 
     def get_operation(self, path, method, registry: ComponentRegistry):
         self.registry = registry
+        self.path = path
+        self.method = method
+
         operation = {}
 
-        operation['operationId'] = self.get_operation_id(path, method)
-        operation['description'] = self.get_description(path, method)
+        operation['operationId'] = self.get_operation_id()
+        operation['description'] = self.get_description()
 
-        parameters = self._get_parameters(path, method)
+        parameters = self._get_parameters()
         if parameters:
             operation['parameters'] = parameters
 
-        tags = self.get_tags(path, method)
+        tags = self.get_tags()
         if tags:
             operation['tags'] = tags
 
-        request_body = self._get_request_body(path, method)
+        request_body = self._get_request_body()
         if request_body:
             operation['requestBody'] = request_body
 
-        auth = self.get_auth(path, method)
+        auth = self.get_auth()
         if auth:
             operation['security'] = auth
 
-        deprecated = self.is_deprecated(path, method)
+        deprecated = self.is_deprecated()
         if deprecated:
             operation['deprecated'] = deprecated
 
-        operation['responses'] = self._get_response_bodies(path, method)
+        operation['responses'] = self._get_response_bodies()
 
         return operation
 
-    def get_override_parameters(self, path, method):
+    def _is_list_view(self, serializer=None):
+        """
+        partially heuristic approach to determine if a view yields an object or a
+        list of objects. used for operationId naming, array building and pagination.
+        defaults to False if all introspection fail.
+        """
+        if serializer is None:
+            serializer = self.get_response_serializers()
+
+        if isinstance(serializer, dict) and serializer:
+            # extract likely main serializer from @extend_schema override
+            serializer = {str(code): s for code, s in serializer.items()}
+            serializer = serializer[min(serializer)]
+
+        if isinstance(serializer, serializers.ListSerializer):
+            return True
+        if is_basic_type(serializer):
+            return False
+        if hasattr(self.view, 'action'):
+            return self.view.action == 'list'
+        # list responses are "usually" only returned by GET
+        if self.method.lower() != 'get':
+            return False
+        # primary key/lookup variable in path is a strong indicator for retrieve
+        if isinstance(self.view, GenericAPIView):
+            lookup_url_kwarg = self.view.lookup_url_kwarg or self.view.lookup_field
+            if lookup_url_kwarg in uritemplate.variables(self.path):
+                return False
+
+        return False
+
+    def get_override_parameters(self):
         """ override this for custom behaviour """
         return []
 
-    def _process_override_parameters(self, path, method):
+    def _process_override_parameters(self):
         result = []
-        for parameter in self.get_override_parameters(path, method):
+        for parameter in self.get_override_parameters():
             if isinstance(parameter, OpenApiParameter):
                 if is_basic_type(parameter.type):
                     schema = build_basic_type(parameter.type)
                 elif is_serializer(parameter.type):
-                    schema = self.resolve_serializer(method, parameter.type).ref
+                    schema = self.resolve_serializer(parameter.type, direction=None).ref
                 else:
                     schema = parameter.type
                 result.append(build_parameter_type(
@@ -174,7 +131,7 @@ class AutoSchema(ViewInspector):
                 ))
             elif is_serializer(parameter):
                 # explode serializer into separate parameters. defaults to QUERY location
-                mapped = self._map_serializer(method, parameter)
+                mapped = self._map_serializer(parameter, direction=None)
                 for property_name, property_schema in mapped['properties'].items():
                     result.append(build_parameter_type(
                         name=property_name,
@@ -186,33 +143,33 @@ class AutoSchema(ViewInspector):
                 warn(f'could not resolve parameter annotation {parameter}. skipping.')
         return result
 
-    def _get_parameters(self, path, method):
+    def _get_parameters(self):
         def dict_helper(parameters):
             return {(p['name'], p['in']): p for p in parameters}
 
-        override_parameters = dict_helper(self._process_override_parameters(path, method))
+        override_parameters = dict_helper(self._process_override_parameters())
         # remove overridden path parameters beforehand so that there are no irrelevant warnings.
         path_variables = [
-            v for v in uritemplate.variables(path) if (v, 'path') not in override_parameters
+            v for v in uritemplate.variables(self.path) if (v, 'path') not in override_parameters
         ]
         parameters = {
             **dict_helper(self._resolve_path_parameters(path_variables)),
-            **dict_helper(self._get_filter_parameters(path, method)),
-            **dict_helper(self._get_pagination_parameters(path, method)),
+            **dict_helper(self._get_filter_parameters()),
+            **dict_helper(self._get_pagination_parameters()),
         }
         # override/add @extend_schema parameters
         for key, parameter in override_parameters.items():
             parameters[key] = parameter
         return sorted(parameters.values(), key=lambda p: p['name'])
 
-    def get_description(self, path, method):
+    def get_description(self):
         """ override this for custom behaviour """
-        action_or_method = getattr(self.view, getattr(self.view, 'action', method.lower()), None)
+        action_or_method = getattr(self.view, getattr(self.view, 'action', self.method.lower()), None)
         view_doc = inspect.getdoc(self.view) or ''
         action_doc = inspect.getdoc(action_or_method) or ''
         return action_doc or view_doc
 
-    def get_auth(self, path, method):
+    def get_auth(self):
         """
         Obtains authentication classes and permissions from view. If authentication
         is known, resolve security requirement for endpoint and security definition for
@@ -231,12 +188,15 @@ class AutoSchema(ViewInspector):
                 )
                 continue
 
-            auths.append(scheme.get_security_requirement(self.view))
+            security_requirements = scheme.get_security_requirement(self)
+            if security_requirements is not None:
+                auths.append(security_requirements)
+
             component = ResolvedComponent(
                 name=scheme.name,
                 type=ResolvedComponent.SECURITY_SCHEMA,
                 object=authenticator.__class__,
-                schema=scheme.get_security_definition(self.view)
+                schema=scheme.get_security_definition(self)
             )
             if component not in self.registry:
                 self.registry.register(component)
@@ -244,47 +204,47 @@ class AutoSchema(ViewInspector):
         perms = [p.__class__ for p in self.view.get_permissions()]
         if permissions.AllowAny in perms:
             auths.append({})
-        elif permissions.IsAuthenticatedOrReadOnly in perms and method not in ('PUT', 'PATCH', 'POST'):
+        elif permissions.IsAuthenticatedOrReadOnly in perms and self.method not in ('PUT', 'PATCH', 'POST'):
             auths.append({})
         return auths
 
-    def get_request_serializer(self, path, method):
+    def get_request_serializer(self):
         """ override this for custom behaviour """
-        return self._get_serializer(path, method)
+        return self._get_serializer()
 
-    def get_response_serializers(self, path, method):
+    def get_response_serializers(self):
         """ override this for custom behaviour """
-        return self._get_serializer(path, method)
+        return self._get_serializer()
 
-    def get_tags(self, path, method) -> typing.List[str]:
+    def get_tags(self) -> typing.List[str]:
         """ override this for custom behaviour """
-        tokenized_path = self._tokenize_path(path)
+        tokenized_path = self._tokenize_path()
         # use first non-parameter path part as tag
         return tokenized_path[:1]
 
-    def get_operation_id(self, path, method):
+    def get_operation_id(self):
         """ override this for custom behaviour """
-        tokenized_path = self._tokenize_path(path)
+        tokenized_path = self._tokenize_path()
         # replace dashes as they can be problematic later in code generation
         tokenized_path = [t.replace('-', '_') for t in tokenized_path]
 
-        if is_list_view(path, method, self.view):
+        if self.method.lower() == 'get' and self._is_list_view():
             action = 'list'
         else:
-            action = self.method_mapping[method.lower()]
+            action = self.method_mapping[self.method.lower()]
 
         return '_'.join(tokenized_path + [action])
 
-    def is_deprecated(self, path, method):
+    def is_deprecated(self):
         """ override this for custom behaviour """
         return False
 
-    def _tokenize_path(self, path):
+    def _tokenize_path(self):
         # remove path prefix
         path = re.sub(
             pattern=spectacular_settings.SCHEMA_PATH_PREFIX,
             repl='',
-            string=path,
+            string=self.path,
             flags=re.IGNORECASE
         )
         # cleanup and tokenize remaining parts.
@@ -331,15 +291,15 @@ class AutoSchema(ViewInspector):
 
         return parameters
 
-    def _get_filter_parameters(self, path, method):
-        if not self._allows_filters(path, method):
+    def _get_filter_parameters(self):
+        if not self._allows_filters():
             return []
         parameters = []
         for filter_backend in self.view.filter_backends:
             parameters += filter_backend().get_schema_operation_parameters(self.view)
         return parameters
 
-    def _allows_filters(self, path, method):
+    def _allows_filters(self):
         """
         Determine whether to include filter Fields in schema.
 
@@ -350,12 +310,12 @@ class AutoSchema(ViewInspector):
             return False
         if hasattr(self.view, 'action'):
             return self.view.action in ["list", "retrieve", "update", "partial_update", "destroy"]
-        return method.lower() in ["get", "put", "patch", "delete"]
+        return self.method.lower() in ["get", "put", "patch", "delete"]
 
-    def _get_pagination_parameters(self, path, method):
+    def _get_pagination_parameters(self):
         view = self.view
 
-        if not is_list_view(path, method, view):
+        if not self._is_list_view():
             return []
 
         paginator = self._get_paginator()
@@ -398,10 +358,9 @@ class AutoSchema(ViewInspector):
 
         if field.__class__ in drf_mapping:
             # use DRF native field resolution - taken from ModelSerializer.get_fields()
-            # TODO maybe init the field with args
-            return self._map_serializer_field(None, drf_mapping[field.__class__]())
-        elif isinstance(field, models.OneToOneField):
-            return self._map_model_field(get_field_from_model(field.model, field.model.id))
+            return self._map_serializer_field(drf_mapping[field.__class__]())
+        elif isinstance(field, models.ForeignKey):
+            return self._map_model_field(field.target_field)
         else:
             warn(
                 f'could not resolve model field "{field}" due to missing mapping.'
@@ -410,24 +369,31 @@ class AutoSchema(ViewInspector):
             )
             return build_basic_type(OpenApiTypes.STR)
 
-    def _map_serializer_field(self, method, field):
+    def _map_serializer_field(self, field):
         if hasattr(field, '_spectacular_annotation'):
             if is_basic_type(field._spectacular_annotation):
                 return build_basic_type(field._spectacular_annotation)
             else:
-                return self._map_serializer_field(method, field._spectacular_annotation)
+                return self._map_serializer_field(field._spectacular_annotation)
+
+        serializer_field_extension = OpenApiSerializerFieldExtension.get_match(field)
+        if serializer_field_extension:
+            return serializer_field_extension.map_serializer_field(self)
+
+        # TODO for now ignore direction while nesting. this would only be relevant
+        #  for nested PATCH, which is likely a very uncommon edge case
 
         # nested serializer
         if isinstance(field, serializers.Serializer):
-            return self.resolve_serializer(method, field).ref
+            return self.resolve_serializer(field, direction=None).ref
 
         # nested serializer with many=True gets automatically replaced with ListSerializer
         if isinstance(field, serializers.ListSerializer):
-            return build_array_type(self.resolve_serializer(method, field.child).ref)
+            return build_array_type(self.resolve_serializer(field.child, direction=None).ref)
 
         # Related fields.
         if isinstance(field, serializers.ManyRelatedField):
-            return build_array_type(self._map_serializer_field(method, field.child_relation))
+            return build_array_type(self._map_serializer_field(field.child_relation))
 
         if isinstance(field, serializers.PrimaryKeyRelatedField):
             # read_only fields do not have a Manager by design. go around and get field
@@ -436,10 +402,7 @@ class AutoSchema(ViewInspector):
             if getattr(field, 'queryset', None) is not None:
                 return self._map_model_field(field.queryset.model._meta.pk)
             else:
-                model = field.parent.Meta.model
-                return self._map_model_field(
-                    get_field_from_model(model, model.id)
-                )
+                return self._map_model_field(field.parent.Meta.model._meta.pk)
 
         if isinstance(field, serializers.StringRelatedField):
             return build_basic_type(OpenApiTypes.STR)
@@ -467,7 +430,7 @@ class AutoSchema(ViewInspector):
             schema = build_array_type({})
             # TODO check this
             if not isinstance(field.child, _UnvalidatedField):
-                map_field = self._map_serializer_field(method, field.child)
+                map_field = self._map_serializer_field(field.child)
                 items = {
                     "type": map_field.get('type')
                 }
@@ -504,9 +467,11 @@ class AutoSchema(ViewInspector):
 
         # DecimalField has multipleOf based on decimal_places
         if isinstance(field, serializers.DecimalField):
-            content = {
-                'type': 'number'
-            }
+            if getattr(field, 'coerce_to_string', api_settings.COERCE_DECIMAL_TO_STRING):
+                content = {**build_basic_type(OpenApiTypes.STR), 'format': 'decimal'}
+            else:
+                content = build_basic_type(OpenApiTypes.DECIMAL)
+
             if field.decimal_places:
                 content['multipleOf'] = float('.' + (field.decimal_places - 1) * '0' + '1')
             if field.max_whole_digits:
@@ -564,16 +529,16 @@ class AutoSchema(ViewInspector):
         if field.min_value:
             content['minimum'] = field.min_value
 
-    def _map_serializer(self, method, serializer):
+    def _map_serializer(self, serializer, direction):
         serializer = force_instance(serializer)
         serializer_extension = OpenApiSerializerExtension.get_match(serializer)
 
         if serializer_extension:
-            return serializer_extension.map_serializer(self, method)
+            return serializer_extension.map_serializer(self, direction)
         else:
-            return self._map_basic_serializer(method, serializer)
+            return self._map_basic_serializer(serializer, direction)
 
-    def _map_basic_serializer(self, method, serializer):
+    def _map_basic_serializer(self, serializer, direction):
         required = []
         properties = {}
 
@@ -584,7 +549,7 @@ class AutoSchema(ViewInspector):
             if field.required:
                 required.append(field.field_name)
 
-            schema = self._map_serializer_field(method, field)
+            schema = self._map_serializer_field(field)
 
             if field.read_only:
                 schema['readOnly'] = True
@@ -593,7 +558,7 @@ class AutoSchema(ViewInspector):
             if field.allow_null:
                 schema['nullable'] = True
             if field.default is not None and field.default != empty and not callable(field.default):
-                schema['default'] = field.default
+                schema['default'] = field.to_representation(field.default)
             if field.help_text:
                 schema['description'] = str(field.help_text)
             self._map_field_validators(field, schema)
@@ -609,7 +574,7 @@ class AutoSchema(ViewInspector):
             'type': 'object',
             'properties': properties
         }
-        if required and method != 'PATCH':
+        if required and (self.method != 'PATCH' or direction == 'response'):
             result['required'] = required
 
         return result
@@ -651,7 +616,7 @@ class AutoSchema(ViewInspector):
         hint = getattr(method, '_spectacular_annotation', None) or typing.get_type_hints(method).get('return')
 
         if is_serializer(hint) or is_field(hint):
-            return self._map_serializer_field(method, force_instance(hint))
+            return self._map_serializer_field(force_instance(hint))
         elif is_basic_type(hint):
             return build_basic_type(hint)
         elif getattr(hint, '__origin__', None) is typing.Union:
@@ -672,10 +637,10 @@ class AutoSchema(ViewInspector):
             return pagination_class()
         return None
 
-    def map_parsers(self, path, method):
+    def map_parsers(self):
         return list(map(attrgetter('media_type'), self.view.parser_classes))
 
-    def map_renderers(self, path, method):
+    def map_renderers(self):
         media_types = []
         for renderer in self.view.renderer_classes:
             # BrowsableAPIRenderer not relevant to OpenAPI spec
@@ -684,32 +649,53 @@ class AutoSchema(ViewInspector):
             media_types.append(renderer.media_type)
         return media_types
 
-    def _get_serializer(self, path, method):
+    def _get_serializer(self):
         view = self.view
         try:
-            # try to circumvent queryset issues with calling get_serializer. if view has NOT
-            # overridden get_serializer, its safe to use get_serializer_class.
-            if view.__class__.get_serializer == GenericAPIView.get_serializer:
-                return view.get_serializer_class()()
-            return view.get_serializer()
+            if isinstance(view, GenericAPIView):
+                # try to circumvent queryset issues with calling get_serializer. if view has NOT
+                # overridden get_serializer, its safe to use get_serializer_class.
+                if view.__class__.get_serializer == GenericAPIView.get_serializer:
+                    return view.get_serializer_class()()
+                return view.get_serializer()
+            elif isinstance(view, APIView):
+                # APIView does not implement the required interface, but be lenient and make
+                # good guesses before giving up and emitting a warning.
+                if callable(getattr(view, 'get_serializer', None)):
+                    return view.get_serializer()
+                elif callable(getattr(view, 'get_serializer_class', None)):
+                    return view.get_serializer_class()()
+                elif hasattr(view, 'serializer_class'):
+                    return view.serializer_class
+                else:
+                    warn(
+                        f'Unable to guess serializer for {view.__class__.__name__}. This is graceful '
+                        f'fallback handling for APIViews. Consider using GenericAPIView as view base '
+                        f'class, if view is under your control. ignoring view for now. '
+                    )
+            else:
+                warn('Encountered unknown view base class. please report this issue. ignoring for now')
         except Exception as exc:
             warn(
                 f'Exception raised while getting serializer from {view.__class__.__name__}. Hint: '
                 f'Is get_serializer_class() returning None or is get_queryset() not working without '
                 f'a request? Ignoring the view for now. (Exception: {exc})'
             )
-            return None
 
-    def _get_request_body(self, path, method):
+    def _get_request_body(self):
         # only unsafe methods can have a body
-        if method not in ('PUT', 'PATCH', 'POST'):
+        if self.method not in ('PUT', 'PATCH', 'POST'):
             return None
 
-        serializer = force_instance(self.get_request_serializer(path, method))
+        serializer = force_instance(self.get_request_serializer())
 
         request_body_required = False
-        if is_serializer(serializer):
-            component = self.resolve_serializer(method, serializer)
+        if isinstance(serializer, serializers.ListSerializer):
+            component = self.resolve_serializer(serializer.child, 'request')
+            schema = build_array_type(component.ref)
+            request_body_required = True
+        elif is_serializer(serializer):
+            component = self.resolve_serializer(serializer, 'request')
             if not component:
                 # serializer is empty so skip content enumeration
                 return None
@@ -722,7 +708,7 @@ class AutoSchema(ViewInspector):
                 return None
         else:
             warn(
-                f'could not resolve request body for {method} {path}. defaulting to generic '
+                f'could not resolve request body for {self.method} {self.path}. defaulting to generic '
                 'free-form object. (maybe annotate a Serializer class?)'
             )
             schema = {
@@ -733,7 +719,7 @@ class AutoSchema(ViewInspector):
 
         request_body = {
             'content': {
-                request_media_types: {'schema': schema} for request_media_types in self.map_parsers(path, method)
+                request_media_types: {'schema': schema} for request_media_types in self.map_parsers()
             }
         }
         if request_body_required:
@@ -741,38 +727,38 @@ class AutoSchema(ViewInspector):
 
         return request_body
 
-    def _get_response_bodies(self, path, method):
-        response_serializers = self.get_response_serializers(path, method)
+    def _get_response_bodies(self):
+        response_serializers = self.get_response_serializers()
 
         if is_serializer(response_serializers) or is_basic_type(response_serializers):
-            if method == 'DELETE':
+            if self.method == 'DELETE':
                 return {'204': {'description': 'No response body'}}
-            return {'200': self._get_response_for_code(path, method, response_serializers)}
+            return {'200': self._get_response_for_code(response_serializers)}
         elif isinstance(response_serializers, dict):
             # custom handling for overriding default return codes with @extend_schema
             return {
-                code: self._get_response_for_code(path, method, serializer)
+                str(code): self._get_response_for_code(serializer)
                 for code, serializer in response_serializers.items()
             }
         else:
             warn(
-                f'could not resolve "{response_serializers}" for {method} {path}. '
+                f'could not resolve "{response_serializers}" for {self.method} {self.path}. '
                 f'Expected either a serializer or some supported override mechanism. '
                 f'defaulting to generic free-form object.'
             )
             schema = build_basic_type(OpenApiTypes.OBJECT)
             schema['description'] = 'Unspecified response body'
-            return {'200': self._get_response_for_code(path, method, schema)}
+            return {'200': self._get_response_for_code(schema)}
 
-    def _get_response_for_code(self, path, method, serializer):
+    def _get_response_for_code(self, serializer):
         serializer = force_instance(serializer)
 
         if not serializer:
             return {'description': 'No response body'}
         elif isinstance(serializer, serializers.ListSerializer):
-            schema = self.resolve_serializer(method, serializer.child).ref
+            schema = self.resolve_serializer(serializer.child, 'response').ref
         elif is_serializer(serializer):
-            component = self.resolve_serializer(method, serializer)
+            component = self.resolve_serializer(serializer, 'response')
             if not component:
                 return {'description': 'No response body'}
             schema = component.ref
@@ -783,15 +769,14 @@ class AutoSchema(ViewInspector):
             schema = serializer
         else:
             warn(
-                f'could not resolve "{serializer}" for {method} {path}. Expected either '
+                f'could not resolve "{serializer}" for {self.method} {self.path}. Expected either '
                 f'a serializer or some supported override mechanism. defaulting to '
                 f'generic free-form object.'
             )
             schema = build_basic_type(OpenApiTypes.OBJECT)
             schema['description'] = 'Unspecified response body'
 
-        if isinstance(serializer, serializers.ListSerializer) or is_list_view(path, method, self.view):
-            # TODO i fear is_list_view is not covering all the cases
+        if self._is_list_view(serializer):
             schema = build_array_type(schema)
             paginator = self._get_paginator()
             if paginator:
@@ -799,7 +784,7 @@ class AutoSchema(ViewInspector):
 
         return {
             'content': {
-                mt: {'schema': schema} for mt in self.map_renderers(path, method)
+                mt: {'schema': schema} for mt in self.map_renderers()
             },
             # Description is required by spec, but descriptions for each response code don't really
             # fit into our model. Description is therefore put into the higher level slots.
@@ -807,7 +792,7 @@ class AutoSchema(ViewInspector):
             'description': ''
         }
 
-    def _get_serializer_name(self, method, serializer):
+    def _get_serializer_name(self, serializer, direction):
         serializer_extension = OpenApiSerializerExtension.get_match(serializer)
         if serializer_extension and serializer_extension.get_name():
             return serializer_extension.get_name()
@@ -816,17 +801,17 @@ class AutoSchema(ViewInspector):
 
         if name.endswith('Serializer'):
             name = name[:-10]
-        if method == 'PATCH' and not serializer.read_only:  # TODO maybe even use serializer.partial
+        if self.method == 'PATCH' and not serializer.read_only and direction == 'request':
             name = 'Patched' + name
 
         return name
 
-    def resolve_serializer(self, method, serializer) -> ResolvedComponent:
+    def resolve_serializer(self, serializer, direction) -> ResolvedComponent:
         assert is_serializer(serializer)
         serializer = force_instance(serializer)
 
         component = ResolvedComponent(
-            name=self._get_serializer_name(method, serializer),
+            name=self._get_serializer_name(serializer, direction),
             type=ResolvedComponent.SCHEMA,
             object=serializer,
         )
@@ -834,12 +819,13 @@ class AutoSchema(ViewInspector):
             return self.registry[component]  # return component with schema
 
         self.registry.register(component)
-        component.schema = self._map_serializer(method, serializer)
-        # 3 cases:
+        component.schema = self._map_serializer(serializer, direction)
+        # 4 cases:
         #   1. polymorphic container component -> use
         #   2. concrete component with properties -> use
         #   3. concrete component without properties -> prob. transactional so discard
-        if 'oneOf' not in component.schema and not component.schema['properties']:
+        #   4. explicit list component -> demultiplexed at usage location so discard
+        if 'oneOf' not in component.schema and not component.schema.get('properties', {}):
             del self.registry[component]
             return ResolvedComponent(None, None)  # sentinel
         return component
